package kr.co.vividnext.sodalive.admin.content.banner

import com.fasterxml.jackson.databind.ObjectMapper
import kr.co.vividnext.sodalive.admin.content.series.AdminContentSeriesRepository
import kr.co.vividnext.sodalive.admin.content.tab.AdminContentMainTabRepository
import kr.co.vividnext.sodalive.aws.s3.S3Uploader
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.main.banner.AudioContentBanner
import kr.co.vividnext.sodalive.content.main.banner.AudioContentBannerType
import kr.co.vividnext.sodalive.event.EventRepository
import kr.co.vividnext.sodalive.member.MemberRepository
import kr.co.vividnext.sodalive.utils.generateFileName
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import org.springframework.web.multipart.MultipartFile

@Service
class AdminContentBannerService(
    private val s3Uploader: S3Uploader,
    private val repository: AdminContentBannerRepository,
    private val memberRepository: MemberRepository,
    private val seriesRepository: AdminContentSeriesRepository,
    private val eventRepository: EventRepository,
    private val contentMainTabRepository: AdminContentMainTabRepository,
    private val objectMapper: ObjectMapper,

    @Value("\${cloud.aws.s3.bucket}")
    private val bucket: String
) {
    @Transactional
    fun createAudioContentMainBanner(image: MultipartFile, requestString: String) {
        val request = objectMapper.readValue(requestString, CreateContentBannerRequest::class.java)
        if (request.type == AudioContentBannerType.CREATOR && request.creatorId == null) {
            throw SodaException("크리에이터를 선택하세요.")
        }

        if (request.type == AudioContentBannerType.SERIES && request.seriesId == null) {
            throw SodaException("시리즈를 선택하세요.")
        }

        if (request.type == AudioContentBannerType.LINK && request.link == null) {
            throw SodaException("링크 url을 입력하세요.")
        }

        if (request.type == AudioContentBannerType.EVENT && request.eventId == null) {
            throw SodaException("이벤트를 선택하세요.")
        }

        val event = if (request.eventId != null && request.eventId > 0) {
            eventRepository.findByIdOrNull(request.eventId)
        } else {
            null
        }

        val creator = if (request.creatorId != null && request.creatorId > 0) {
            memberRepository.findByIdOrNull(request.creatorId)
        } else {
            null
        }

        val series = if (request.seriesId != null && request.seriesId > 0) {
            seriesRepository.findByIdOrNull(request.seriesId)
        } else {
            null
        }

        val tab = if (request.tabId !== null) {
            contentMainTabRepository.findByIdOrNull(request.tabId)
        } else {
            null
        }

        val audioContentBanner = AudioContentBanner(type = request.type)
        audioContentBanner.link = request.link
        audioContentBanner.isAdult = request.isAdult
        audioContentBanner.event = event
        audioContentBanner.creator = creator
        audioContentBanner.series = series
        audioContentBanner.tab = tab
        repository.save(audioContentBanner)

        val fileName = generateFileName()
        val imagePath = s3Uploader.upload(
            inputStream = image.inputStream,
            bucket = bucket,
            filePath = "audio_content_banner/${audioContentBanner.id}/$fileName"
        )
        audioContentBanner.thumbnailImage = imagePath
    }

    @Transactional
    fun updateAudioContentMainBanner(image: MultipartFile?, requestString: String) {
        val request = objectMapper.readValue(requestString, UpdateContentBannerRequest::class.java)
        val audioContentBanner = repository.findByIdOrNull(request.id)
            ?: throw SodaException("잘못된 요청입니다.")

        if (image != null) {
            val fileName = generateFileName()
            val imagePath = s3Uploader.upload(
                inputStream = image.inputStream,
                bucket = bucket,
                filePath = "audio_content_banner/${audioContentBanner.id}/$fileName"
            )
            audioContentBanner.thumbnailImage = imagePath
        }

        if (request.isAdult != null) {
            audioContentBanner.isAdult = request.isAdult
        }

        if (request.isActive != null) {
            audioContentBanner.isActive = request.isActive
        }

        if (request.type != null) {
            audioContentBanner.creator = null
            audioContentBanner.event = null
            audioContentBanner.link = null
            audioContentBanner.series = null

            when (request.type) {
                AudioContentBannerType.EVENT -> {
                    if (request.eventId != null) {
                        val event = eventRepository.findByIdOrNull(request.eventId)
                            ?: throw SodaException("이벤트를 선택하세요.")

                        audioContentBanner.event = event
                    } else {
                        throw SodaException("이벤트를 선택하세요.")
                    }
                }

                AudioContentBannerType.CREATOR -> {
                    if (request.creatorId != null) {
                        val creator = memberRepository.findByIdOrNull(request.creatorId)
                            ?: throw SodaException("크리에이터를 선택하세요.")

                        audioContentBanner.creator = creator
                    } else {
                        throw SodaException("크리에이터를 선택하세요.")
                    }
                }

                AudioContentBannerType.LINK -> {
                    if (request.link != null) {
                        audioContentBanner.link = request.link
                    } else {
                        throw SodaException("링크 url을 입력하세요.")
                    }
                }

                AudioContentBannerType.SERIES -> {
                    if (request.seriesId != null) {
                        val series = seriesRepository.findByIdOrNull(request.seriesId)
                            ?: throw SodaException("시리즈를 선택하세요.")

                        audioContentBanner.series = series
                    } else {
                        throw SodaException("시리즈를 선택하세요.")
                    }
                }
            }

            audioContentBanner.type = request.type
        }

        if (request.tabId !== null) {
            audioContentBanner.tab = contentMainTabRepository.findByIdOrNull(request.tabId)
        }
    }

    @Transactional
    fun updateBannerOrders(ids: List<Long>) {
        for (index in ids.indices) {
            val tag = repository.findByIdOrNull(ids[index])

            if (tag != null) {
                tag.orders = index + 1
            }
        }
    }

    fun getAudioContentMainBannerList(tabId: Long?): List<GetAdminContentBannerResponse> {
        return repository.getAudioContentMainBannerList(tabId = tabId ?: 1)
    }
}
