package kr.co.vividnext.sodalive.content.main

import kr.co.vividnext.sodalive.content.AudioContentRepository
import kr.co.vividnext.sodalive.content.ContentType
import kr.co.vividnext.sodalive.content.main.banner.AudioContentBannerType
import kr.co.vividnext.sodalive.content.main.banner.GetAudioContentBannerResponse
import kr.co.vividnext.sodalive.content.main.curation.GetAudioContentCurationResponse
import kr.co.vividnext.sodalive.content.theme.AudioContentThemeQueryRepository
import kr.co.vividnext.sodalive.event.EventItem
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.block.BlockMemberRepository
import org.springframework.beans.factory.annotation.Value
import org.springframework.cache.annotation.Cacheable
import org.springframework.data.domain.Pageable
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
class AudioContentMainService(
    private val repository: AudioContentRepository,
    private val blockMemberRepository: BlockMemberRepository,
    private val audioContentThemeRepository: AudioContentThemeQueryRepository,

    @Value("\${cloud.aws.cloud-front.host}")
    private val imageHost: String
) {
    @Transactional(readOnly = true)
    @Cacheable(cacheNames = ["default"], key = "'themeList:' + ':' + #isAdult")
    fun getThemeList(isAdult: Boolean): List<String> {
        return audioContentThemeRepository.getActiveThemeOfContent(isAdult = isAdult)
    }

    @Transactional(readOnly = true)
    fun getNewContentByTheme(
        theme: String,
        isAdultContentVisible: Boolean,
        contentType: ContentType,
        member: Member,
        pageable: Pageable
    ): List<GetAudioContentMainItem> {
        return repository.findByTheme(
            memberId = member.id!!,
            theme = if (theme.isNotBlank()) {
                listOf(theme)
            } else {
                emptyList()
            },
            isAdult = member.auth != null && isAdultContentVisible,
            contentType = contentType,
            offset = pageable.offset,
            limit = pageable.pageSize.toLong()
        )
    }

    @Transactional(readOnly = true)
    fun getNewContentFor2WeeksByTheme(
        isFree: Boolean,
        theme: String,
        isAdultContentVisible: Boolean,
        contentType: ContentType,
        member: Member,
        pageable: Pageable
    ): GetNewContentAllResponse {
        val isAdult = member.auth != null && isAdultContentVisible
        val themeList = if (theme.isBlank() || theme.trim() == "전체") {
            audioContentThemeRepository.getActiveThemeOfContent(isAdult = isAdult)
        } else {
            listOf(theme)
        }

        val totalCount = repository.totalCountNewContentFor2Weeks(
            isFree,
            themeList,
            memberId = member.id!!,
            isAdult = isAdult,
            contentType = contentType
        )
        val items = repository.findByThemeFor2Weeks(
            isFree,
            cloudfrontHost = imageHost,
            memberId = member.id!!,
            theme = themeList,
            isAdult = isAdult,
            contentType = contentType,
            offset = pageable.offset,
            limit = pageable.pageSize.toLong()
        )
            .filter { !blockMemberRepository.isBlocked(blockedMemberId = member.id!!, memberId = it.creatorId) }

        return GetNewContentAllResponse(totalCount, items)
    }

    @Transactional(readOnly = true)
    @Cacheable(cacheNames = ["default"], key = "'newContentUploadCreatorList:' + #memberId + ':' + #isAdult")
    fun getNewContentUploadCreatorList(memberId: Long, isAdult: Boolean): List<ContentCreatorResponse> {
        return repository.getNewContentUploadCreatorList(
            cloudfrontHost = imageHost,
            isAdult = isAdult
        )
            .asSequence()
            .filter { !blockMemberRepository.isBlocked(blockedMemberId = memberId, memberId = it.creatorId) }
            .toList()
    }

    @Transactional(readOnly = true)
    fun getAudioContentMainBannerList(memberId: Long, isAdult: Boolean) =
        repository.getAudioContentMainBannerList(isAdult = isAdult)
            .asSequence()
            .filter {
                if (it.type == AudioContentBannerType.CREATOR && it.creator != null) {
                    !blockMemberRepository.isBlocked(blockedMemberId = memberId, memberId = it.creator!!.id!!)
                } else if (it.type == AudioContentBannerType.SERIES && it.series != null) {
                    !blockMemberRepository.isBlocked(blockedMemberId = memberId, memberId = it.series!!.member!!.id!!)
                } else {
                    true
                }
            }
            .map {
                GetAudioContentBannerResponse(
                    type = it.type,
                    thumbnailImageUrl = "$imageHost/${it.thumbnailImage}",
                    eventItem = if (it.type == AudioContentBannerType.EVENT && it.event != null) {
                        EventItem(
                            id = it.event!!.id!!,
                            thumbnailImageUrl = if (!it.event!!.thumbnailImage.startsWith("https://")) {
                                "$imageHost/${it.event!!.thumbnailImage}"
                            } else {
                                it.event!!.thumbnailImage
                            },
                            detailImageUrl = if (
                                it.event!!.detailImage != null &&
                                !it.event!!.detailImage!!.startsWith("https://")
                            ) {
                                "$imageHost/${it.event!!.detailImage}"
                            } else {
                                it.event!!.detailImage
                            },
                            popupImageUrl = null,
                            link = it.event!!.link
                        )
                    } else {
                        null
                    },
                    creatorId = if (it.type == AudioContentBannerType.CREATOR && it.creator != null) {
                        it.creator!!.id
                    } else {
                        null
                    },
                    seriesId = if (it.type == AudioContentBannerType.SERIES && it.series != null) {
                        it.series!!.id
                    } else {
                        null
                    },
                    link = it.link
                )
            }
            .toList()

    @Transactional(readOnly = true)
    @Cacheable(
        cacheNames = ["default"],
        key = "'getAudioContentCurationListWithPaging:' + #memberId + ':' + #isAdult + ':' + #contentType" +
            "+ ':' + #offset + ':' + #limit"
    )
    fun getAudioContentCurationListWithPaging(
        memberId: Long,
        isAdult: Boolean,
        contentType: ContentType,
        offset: Long,
        limit: Long
    ) = repository.getAudioContentCurationList(isAdult = isAdult, offset = offset, limit = limit)
        .map {
            GetAudioContentCurationResponse(
                curationId = it.id!!,
                title = it.title,
                description = it.description,
                contents = repository.findAudioContentByCurationId(
                    curationId = it.id!!,
                    isAdult = isAdult,
                    contentType = contentType
                )
                    .filter { content ->
                        !blockMemberRepository.isBlocked(blockedMemberId = memberId, memberId = content.creatorId)
                    }
            )
        }
        .filter { it.contents.isNotEmpty() }
}
