package kr.co.vividnext.sodalive.content.main.tab

import com.querydsl.core.types.dsl.Expressions
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.main.ContentCreatorResponse
import kr.co.vividnext.sodalive.content.main.QContentCreatorResponse
import kr.co.vividnext.sodalive.content.theme.QAudioContentTheme.audioContentTheme
import kr.co.vividnext.sodalive.member.MemberRole
import kr.co.vividnext.sodalive.member.QMember.member
import kr.co.vividnext.sodalive.member.block.QBlockMember.blockMember
import org.springframework.beans.factory.annotation.Value
import org.springframework.stereotype.Repository

@Repository
class AudioContentMainTabRepository(
    private val queryFactory: JPAQueryFactory,

    @Value("\${cloud.aws.cloud-front.host}")
    private val imageHost: String
) {
    fun findCreatorByThemeContent(
        memberId: Long,
        theme: String,
        minCount: Int
    ): List<ContentCreatorResponse> {
        val blockMemberCondition = blockMember.member.id.eq(member.id)
            .and(blockMember.isActive.isTrue)
            .and(blockMember.blockedMember.id.eq(memberId))

        val where = member.isActive.isTrue
            .and(member.role.eq(MemberRole.CREATOR))
            .and(audioContent.isActive.isTrue)
            .and(audioContent.price.gt(0))
            .and(audioContent.duration.isNotNull)
            .and(audioContent.limited.isNull)
            .and(audioContentTheme.isActive.isTrue)
            .and(audioContentTheme.theme.eq(theme))
            .and(blockMember.id.isNull)

        return queryFactory
            .select(
                QContentCreatorResponse(
                    member.id,
                    member.nickname,
                    member.profileImage.prepend("/").prepend(imageHost)
                )
            )
            .from(member)
            .innerJoin(audioContent).on(member.id.eq(audioContent.member.id))
            .innerJoin(audioContent.theme, audioContentTheme)
            .leftJoin(blockMember).on(blockMemberCondition)
            .where(where)
            .groupBy(member.id)
            .having(audioContent.id.count().goe(minCount))
            .orderBy(
                Expressions.numberTemplate(Double::class.java, "function('rand')").asc()
            )
            .offset(0)
            .limit(20)
            .fetch()
    }

    fun findCreatorWithHasFreeContent(memberId: Long, minCount: Int): List<ContentCreatorResponse> {
        val blockMemberCondition = blockMember.member.id.eq(member.id)
            .and(blockMember.isActive.isTrue)
            .and(blockMember.blockedMember.id.eq(memberId))

        val where = member.isActive.isTrue
            .and(member.role.eq(MemberRole.CREATOR))
            .and(audioContent.isActive.isTrue)
            .and(audioContent.price.loe(0))
            .and(audioContent.duration.isNotNull)
            .and(audioContent.limited.isNull)
            .and(blockMember.id.isNull)

        return queryFactory
            .select(
                QContentCreatorResponse(
                    member.id,
                    member.nickname,
                    member.profileImage.prepend("/").prepend(imageHost)
                )
            )
            .from(member)
            .innerJoin(audioContent).on(member.id.eq(audioContent.member.id))
            .leftJoin(blockMember).on(blockMemberCondition)
            .where(where)
            .groupBy(member.id)
            .having(audioContent.id.count().goe(minCount))
            .orderBy(
                Expressions.numberTemplate(Double::class.java, "function('rand')").asc()
            )
            .offset(0)
            .limit(20)
            .fetch()
    }
}
