package kr.co.vividnext.sodalive.content.main.tab.content

import kr.co.vividnext.sodalive.content.AudioContentRepository
import kr.co.vividnext.sodalive.content.ContentType
import kr.co.vividnext.sodalive.content.main.GetAudioContentMainItem
import kr.co.vividnext.sodalive.content.main.GetAudioContentRankingItem
import kr.co.vividnext.sodalive.content.main.banner.AudioContentBannerService
import kr.co.vividnext.sodalive.content.main.curation.AudioContentCurationQueryRepository
import kr.co.vividnext.sodalive.content.main.tab.GetContentCurationResponse
import kr.co.vividnext.sodalive.content.theme.AudioContentThemeQueryRepository
import kr.co.vividnext.sodalive.event.EventService
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.rank.RankingService
import org.springframework.stereotype.Service
import java.time.LocalDateTime

@Service
class AudioContentMainTabContentService(
    private val bannerService: AudioContentBannerService,
    private val audioContentRepository: AudioContentRepository,
    private val audioContentThemeRepository: AudioContentThemeQueryRepository,
    private val rankingService: RankingService,
    private val eventService: EventService,
    private val tagCurationService: ContentMainTabTagCurationService,
    private val curationRepository: AudioContentCurationQueryRepository
) {
    fun fetchData(
        isAdultContentVisible: Boolean,
        contentType: ContentType,
        member: Member
    ): GetContentMainTabContentResponse {
        val memberId = member.id!!
        val isAdult = member.auth != null
        val tabId = 3L

        // 단편 배너
        val contentBannerList = bannerService.getBannerList(
            tabId = tabId,
            memberId = memberId,
            isAdult = isAdult
        )

        // 새로운 단편 테마
        val themeOfContentList = audioContentThemeRepository.getActiveThemeOfContent(isAdult = isAdult)
            .filter {
                it != "오디오북" &&
                    it != "모닝콜" &&
                    it != "알람" &&
                    it != "슬립콜" &&
                    it != "다시듣기" &&
                    it != "ASMR" &&
                    it != "릴레이" &&
                    it != "챌린지" &&
                    it != "자기소개"
            }

        // 새로운 단편
        val newContentList = audioContentRepository.findByTheme(
            memberId = member.id!!,
            theme = themeOfContentList,
            isAdult = member.auth != null,
            contentType = ContentType.ALL,
            offset = 0,
            limit = 10
        )

        // 일간 랭킹
        val currentDateTime = LocalDateTime.now()
        val dailyRankingStartDate = currentDateTime
            .withHour(15)
            .withMinute(0)
            .withSecond(0)
            .minusDays(2)
        val dailyRankingEndDate = dailyRankingStartDate
            .plusDays(1)

        val rankContentList = rankingService.getContentRanking(
            memberId = memberId,
            isAdult = isAdult,
            startDate = dailyRankingStartDate,
            endDate = dailyRankingEndDate
        )

        // 이벤트 배너
        val eventBannerList = eventService.getEventList(isAdult = isAdult)

        val contentRankCreatorList = rankingService.fetchCreatorBySellContentCountRankTop20(
            memberId = member.id!!,
            startDate = dailyRankingStartDate,
            endDate = dailyRankingEndDate
        )

        val salesCountRankContentList = if (contentRankCreatorList.isNotEmpty()) {
            rankingService.fetchCreatorContentBySalesCountTop4(
                creatorId = contentRankCreatorList[0].creatorId,
                isAdult = member.auth != null
            )
        } else {
            emptyList()
        }

        val tagList = if (isAdult) {
            tagCurationService.getTagList(isAdult = isAdult)
        } else {
            emptyList()
        }

        val tagCurationContentList = if (tagList.isNotEmpty()) {
            tagCurationService.getTagCurationContentList(memberId = memberId, tag = tagList[0])
        } else {
            emptyList()
        }

        val curationList = curationRepository.findByContentMainTabId(tabId = tabId, isAdult = isAdult)
            .map {
                GetContentCurationResponse(
                    title = it.title,
                    items = audioContentRepository.findAudioContentByCurationIdV2(
                        curationId = it.id!!,
                        memberId = memberId,
                        isAdult = isAdult
                    )
                )
            }

        return GetContentMainTabContentResponse(
            bannerList = contentBannerList,
            contentThemeList = themeOfContentList,
            newContentList = newContentList,
            rankSortTypeList = listOf("매출", "댓글", "좋아요"),
            rankContentList = rankContentList,
            contentRankCreatorList = contentRankCreatorList,
            salesCountRankContentList = salesCountRankContentList,
            eventBannerList = eventBannerList,
            tagList = tagList,
            tagCurationContentList = tagCurationContentList,
            curationList = curationList
        )
    }

    fun getAudioContentRanking(
        memberId: Long,
        isAdult: Boolean,
        sortType: String
    ): List<GetAudioContentRankingItem> {
        val currentDateTime = LocalDateTime.now()
        val dailyRankingStartDate = currentDateTime
            .withHour(15)
            .withMinute(0)
            .withSecond(0)
            .minusDays(2)
        val dailyRankingEndDate = dailyRankingStartDate
            .plusDays(1)

        return rankingService.getContentRanking(
            memberId = memberId,
            isAdult = isAdult,
            startDate = dailyRankingStartDate,
            endDate = dailyRankingEndDate,
            sortType = sortType
        )
    }

    fun getNewContentByTheme(
        theme: String,
        isAdultContentVisible: Boolean,
        contentType: ContentType,
        member: Member
    ): List<GetAudioContentMainItem> {
        val isAdult = member.auth != null && isAdultContentVisible

        val themeList = if (theme.isBlank()) {
            audioContentThemeRepository.getActiveThemeOfContent(isAdult = isAdult)
                .filter {
                    it != "오디오북" &&
                        it != "모닝콜" &&
                        it != "알람" &&
                        it != "슬립콜" &&
                        it != "다시듣기" &&
                        it != "ASMR" &&
                        it != "릴레이" &&
                        it != "챌린지" &&
                        it != "자기소개"
                }
        } else {
            listOf(theme)
        }

        return audioContentRepository.findByTheme(
            memberId = member.id!!,
            theme = themeList,
            isAdult = isAdult,
            contentType = contentType,
            offset = 0,
            limit = 10
        )
    }

    fun getPopularContentByCreator(creatorId: Long, isAdult: Boolean): List<GetAudioContentRankingItem> {
        return rankingService.fetchCreatorContentBySalesCountTop4(
            creatorId = creatorId,
            isAdult = isAdult
        )
    }

    fun getRecommendedContentByTag(memberId: Long, tag: String): List<GetAudioContentMainItem> {
        return tagCurationService.getTagCurationContentList(memberId = memberId, tag = tag)
    }
}
