package kr.co.vividnext.sodalive.content.main.tab.series

import kr.co.vividnext.sodalive.content.main.banner.AudioContentBannerService
import kr.co.vividnext.sodalive.content.main.curation.AudioContentCurationQueryRepository
import kr.co.vividnext.sodalive.content.main.tab.RecommendSeriesRepository
import kr.co.vividnext.sodalive.content.series.ContentSeriesService
import kr.co.vividnext.sodalive.content.series.GetSeriesListResponse
import kr.co.vividnext.sodalive.event.EventService
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.rank.RankingService
import org.springframework.stereotype.Service
import java.time.DayOfWeek
import java.time.LocalDateTime
import java.time.temporal.TemporalAdjusters

@Service
class AudioContentMainTabSeriesService(
    private val bannerService: AudioContentBannerService,
    private val seriesService: ContentSeriesService,
    private val rankingService: RankingService,
    private val recommendSeriesRepository: RecommendSeriesRepository,
    private val eventService: EventService,
    private val curationRepository: AudioContentCurationQueryRepository
) {
    fun fetchData(member: Member): GetContentMainTabSeriesResponse {
        val isAdult = member.auth != null
        val memberId = member.id!!

        // 메인 배너 (시리즈)
        val contentBannerList = bannerService.getBannerList(
            tabId = 2,
            memberId = memberId,
            isAdult = isAdult
        )

        val originalAudioDrama = seriesService.getOriginalAudioDramaList(
            memberId = memberId,
            isAdult = isAdult,
            offset = 0,
            limit = 20
        )

        // 일간 랭킹
        val currentDateTime = LocalDateTime.now()
        val dailyRankingStartDate = currentDateTime
            .withHour(15)
            .withMinute(0)
            .withSecond(0)
            .minusDays(2)
        val dailyRankingEndDate = dailyRankingStartDate
            .plusDays(1)

        val rankSeriesList = rankingService.getSeriesRanking(
            memberId = memberId,
            isAdult = isAdult,
            startDate = dailyRankingStartDate,
            endDate = dailyRankingEndDate
        )

        // 시리즈 장르
        val genreList = seriesService.getGenreList(isAdult = isAdult)

        // 장르별 추천 시리즈
        val recommendSeriesList = if (genreList.isNotEmpty()) {
            rankingService.getSeriesAllRankingByGenre(
                memberId = memberId,
                isAdult = isAdult,
                genreId = genreList[0].id
            )
        } else {
            emptyList()
        }

        // 새로운 시리즈
        val newSeriesList = recommendSeriesRepository.getNewSeriesList(isAdult = isAdult)

        // 완결 시리즈 월간 랭킹
        val monthlyRankingStartDate = currentDateTime
            .withDayOfMonth(1)
            .withHour(15)
            .withMinute(0)
            .withSecond(0)
            .minusDays(1)
        val monthlyRankingEndDate = monthlyRankingStartDate
            .plusMonths(1)

        val rankCompleteSeriesList = rankingService.getCompleteSeriesRanking(
            memberId = memberId,
            isAdult = isAdult,
            startDate = monthlyRankingStartDate,
            endDate = monthlyRankingEndDate
        )

        val startDate = currentDateTime
            .withHour(15)
            .withMinute(0)
            .withSecond(0)
            .minusWeeks(1)
            .with(TemporalAdjusters.previousOrSame(DayOfWeek.MONDAY))
        val endDate = startDate
            .plusDays(6)

        val seriesRankCreatorList = rankingService.fetchCreatorBySeriesRevenueRankTop20(
            memberId = memberId,
            startDate = startDate.minusDays(1),
            endDate = endDate
        )

        val salesRankContentList = if (seriesRankCreatorList.isNotEmpty()) {
            rankingService.fetchCreatorSeriesBySales(
                creatorId = seriesRankCreatorList[0].creatorId,
                isAdult = isAdult
            )
        } else {
            emptyList()
        }

        // 이벤트 배너
        val eventBannerList = eventService.getEventList(isAdult = isAdult)

        // 큐레이션
        val curationList = curationRepository.findByContentMainTabId(tabId = 2, isAdult = isAdult)
            .map {
                GetSeriesCurationResponse(
                    title = it.title,
                    items = seriesService.fetchSeriesByCurationId(
                        curationId = it.id!!,
                        memberId = memberId,
                        isAdult = isAdult
                    )
                )
            }

        return GetContentMainTabSeriesResponse(
            contentBannerList = contentBannerList,
            originalAudioDrama = if (originalAudioDrama.size >= 3) {
                originalAudioDrama
            } else {
                emptyList()
            },
            rankSeriesList = rankSeriesList,
            genreList = genreList,
            recommendSeriesList = recommendSeriesList,
            newSeriesList = newSeriesList,
            rankCompleteSeriesList = rankCompleteSeriesList,
            seriesRankCreatorList = seriesRankCreatorList,
            recommendSeriesByChannel = salesRankContentList,
            eventBannerList = eventBannerList,
            curationList = curationList
        )
    }

    fun getOriginalAudioDramaList(
        memberId: Long,
        isAdult: Boolean,
        offset: Long,
        limit: Long
    ): GetSeriesListResponse {
        val totalCount = seriesService.getOriginalAudioDramaTotalCount(memberId, isAdult)
        val items = seriesService.getOriginalAudioDramaList(
            memberId = memberId,
            isAdult = isAdult,
            offset = offset,
            limit = limit
        )

        return GetSeriesListResponse(totalCount, items)
    }

    fun getRankMonthlyCompletedSeriesList(
        memberId: Long,
        isAdult: Boolean,
        offset: Long,
        limit: Long
    ): GetSeriesListResponse {
        val monthlyRankingStartDate = LocalDateTime.now()
            .withDayOfMonth(1)
            .withHour(15)
            .withMinute(0)
            .withSecond(0)
            .minusDays(1)
        val monthlyRankingEndDate = monthlyRankingStartDate
            .plusMonths(1)

        val totalCount = rankingService.getCompleteSeriesRankingTotalCount(
            memberId = memberId,
            isAdult = isAdult
        )

        val items = rankingService.getCompleteSeriesRanking(
            memberId = memberId,
            isAdult = isAdult,
            startDate = monthlyRankingStartDate,
            endDate = monthlyRankingEndDate,
            offset = offset,
            limit = limit
        )

        return GetSeriesListResponse(totalCount, items)
    }

    fun getRecommendSeriesListByGenre(
        genreId: Long,
        memberId: Long,
        isAdult: Boolean
    ): List<GetSeriesListResponse.SeriesListItem> {
        return rankingService.getSeriesAllRankingByGenre(
            memberId = memberId,
            isAdult = isAdult,
            genreId = genreId
        )
    }

    fun getRecommendSeriesByCreator(creatorId: Long, isAdult: Boolean): List<GetSeriesListResponse.SeriesListItem> {
        return rankingService.fetchCreatorSeriesBySales(
            creatorId = creatorId,
            isAdult = isAdult
        )
    }
}
