package kr.co.vividnext.sodalive.creator.admin.content

import com.amazonaws.services.s3.model.ObjectMetadata
import com.fasterxml.jackson.databind.ObjectMapper
import kr.co.vividnext.sodalive.aws.cloudfront.AudioContentCloudFront
import kr.co.vividnext.sodalive.aws.s3.S3Uploader
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.ContentPriceChangeLog
import kr.co.vividnext.sodalive.content.ContentPriceChangeLogRepository
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.utils.generateFileName
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.domain.Pageable
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import org.springframework.web.multipart.MultipartFile

@Service
class CreatorAdminContentService(
    private val repository: CreatorAdminContentRepository,
    private val contentPriceChangeLogRepository: ContentPriceChangeLogRepository,
    private val audioContentCloudFront: AudioContentCloudFront,
    private val objectMapper: ObjectMapper,
    private val s3Uploader: S3Uploader,

    @Value("\${cloud.aws.s3.bucket}")
    private val bucket: String,

    @Value("\${cloud.aws.cloud-front.host}")
    private val coverImageHost: String
) {
    fun getAudioContentList(pageable: Pageable, member: Member): GetCreatorAdminContentListResponse {
        val totalCount = repository.getAudioContentTotalCount(memberId = member.id!!)
        val audioContentAndThemeList = repository.getAudioContentList(
            memberId = member.id!!,
            offset = pageable.offset,
            limit = pageable.pageSize.toLong()
        )

        val audioContentList = audioContentAndThemeList
            .asSequence()
            .map {
                val tags = repository
                    .getHashTagList(audioContentId = it.audioContentId)
                    .joinToString(" ") { tag -> tag }
                it.tags = tags
                it
            }
            .map {
                it.contentUrl = audioContentCloudFront.generateSignedURL(
                    resourcePath = it.contentUrl,
                    expirationTime = 1000 * 60 * 60 * (it.remainingTime.split(":")[0].toLong() + 2)
                )
                it
            }
            .map {
                it.coverImageUrl = "$coverImageHost/${it.coverImageUrl}"
                it
            }
            .toList()

        return GetCreatorAdminContentListResponse(totalCount, audioContentList)
    }

    fun searchAudioContent(searchWord: String, member: Member, pageable: Pageable): GetCreatorAdminContentListResponse {
        if (searchWord.length < 2) throw SodaException("2글자 이상 입력하세요.")
        val totalCount = repository.getAudioContentTotalCount(
            memberId = member.id!!,
            searchWord
        )
        val audioContentAndThemeList = repository.getAudioContentList(
            memberId = member.id!!,
            offset = pageable.offset,
            limit = pageable.pageSize.toLong(),
            searchWord = searchWord
        )

        val audioContentList = audioContentAndThemeList
            .asSequence()
            .map {
                val tags = repository
                    .getHashTagList(audioContentId = it.audioContentId)
                    .joinToString(" ") { tag -> tag }
                it.tags = tags
                it
            }
            .map {
                it.contentUrl = audioContentCloudFront.generateSignedURL(
                    resourcePath = it.contentUrl,
                    expirationTime = 1000 * 60 * 60 * (it.remainingTime.split(":")[0].toLong() + 2)
                )
                it
            }
            .map {
                it.coverImageUrl = "$coverImageHost/${it.coverImageUrl}"
                it
            }
            .toList()

        return GetCreatorAdminContentListResponse(totalCount, audioContentList)
    }

    @Transactional
    fun updateAudioContent(coverImage: MultipartFile?, requestString: String, member: Member) {
        val request = objectMapper.readValue(requestString, UpdateCreatorAdminContentRequest::class.java)
        val audioContent = repository.getAudioContent(memberId = member.id!!, audioContentId = request.id)
            ?: throw SodaException("잘못된 콘텐츠 입니다.")

        if (coverImage != null) {
            val metadata = ObjectMetadata()
            metadata.contentLength = coverImage.size

            val fileName = generateFileName()
            val imagePath = s3Uploader.upload(
                inputStream = coverImage.inputStream,
                bucket = bucket,
                filePath = "audio_content_cover/${request.id}/$fileName",
                metadata = metadata
            )
            audioContent.coverImage = imagePath
        }

        if (request.isActive != null) {
            if (!request.isActive) {
                audioContent.releaseDate = null
            }
            audioContent.isActive = request.isActive
        }

        if (request.isAdult != null) {
            audioContent.isAdult = request.isAdult
        }

        if (request.isCommentAvailable != null) {
            audioContent.isCommentAvailable = request.isCommentAvailable
        }

        if (request.title != null) {
            audioContent.title = request.title
        }

        if (request.detail != null) {
            audioContent.detail = request.detail
        }

        if (request.price != null) {
            if (request.price < 5) throw SodaException("콘텐츠의 최소금액은 5캔 입니다.")

            val contentPriceChangeLog = ContentPriceChangeLog(prevPrice = audioContent.price)
            contentPriceChangeLog.audioContent = audioContent
            contentPriceChangeLogRepository.save(contentPriceChangeLog)

            audioContent.price = request.price
        }
    }
}
