package kr.co.vividnext.sodalive.explorer

import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.AudioContentService
import kr.co.vividnext.sodalive.content.SortType
import kr.co.vividnext.sodalive.content.series.ContentSeriesService
import kr.co.vividnext.sodalive.explorer.follower.GetFollowerListResponse
import kr.co.vividnext.sodalive.explorer.follower.GetFollowerListResponseItem
import kr.co.vividnext.sodalive.explorer.profile.ChannelNotice
import kr.co.vividnext.sodalive.explorer.profile.ChannelNoticeRepository
import kr.co.vividnext.sodalive.explorer.profile.CreatorCheers
import kr.co.vividnext.sodalive.explorer.profile.CreatorCheersRepository
import kr.co.vividnext.sodalive.explorer.profile.PostWriteCheersRequest
import kr.co.vividnext.sodalive.explorer.profile.PutWriteCheersRequest
import kr.co.vividnext.sodalive.explorer.profile.creatorCommunity.CreatorCommunityService
import kr.co.vividnext.sodalive.fcm.FcmEvent
import kr.co.vividnext.sodalive.fcm.FcmEventType
import kr.co.vividnext.sodalive.live.room.detail.GetRoomDetailUser
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.MemberRole
import kr.co.vividnext.sodalive.member.MemberService
import org.springframework.beans.factory.annotation.Value
import org.springframework.context.ApplicationEventPublisher
import org.springframework.data.domain.Pageable
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import java.time.DayOfWeek
import java.time.LocalDate
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter
import java.time.temporal.TemporalAdjusters
import kotlin.random.Random

@Service
@Transactional(readOnly = true)
class ExplorerService(
    private val memberService: MemberService,
    private val audioContentService: AudioContentService,
    private val queryRepository: ExplorerQueryRepository,
    private val cheersRepository: CreatorCheersRepository,
    private val noticeRepository: ChannelNoticeRepository,
    private val communityService: CreatorCommunityService,
    private val seriesService: ContentSeriesService,

    private val applicationEventPublisher: ApplicationEventPublisher,

    @Value("\${cloud.aws.cloud-front.host}")
    private val cloudFrontHost: String
) {
    fun getCreatorRank(memberId: Long): GetExplorerSectionResponse {
        val creatorRankings = queryRepository
            .getCreatorRankings()
            .filter { !memberService.isBlocked(blockedMemberId = memberId, memberId = it.id!!) }
            .map { it.toExplorerSectionCreator(cloudFrontHost) }

        val currentDateTime = LocalDateTime.now()
        val lastMonday = currentDateTime
            .minusWeeks(1)
            .with(TemporalAdjusters.previousOrSame(DayOfWeek.MONDAY))
        val lastSunday = lastMonday
            .plusDays(6)

        val startDateFormatter = DateTimeFormatter.ofPattern("yyyy년 MM월 dd일")
        val endDateFormatter = DateTimeFormatter.ofPattern("MM월 dd일")

        val formattedLastMonday = lastMonday.format(startDateFormatter)
        val formattedLastSunday = lastSunday.format(endDateFormatter)

        return GetExplorerSectionResponse(
            title = "인기 크리에이터",
            coloredTitle = "인기",
            color = "FF5C49",
            desc = "$formattedLastMonday ~ $formattedLastSunday",
            creators = creatorRankings
        )
    }

    fun getExplorer(member: Member, growthRankingCreatorsLimit: Long = 20): GetExplorerResponse {
        val sections = mutableListOf<GetExplorerSectionResponse>()

        // 인기 크리에이터
        val creatorRankings = queryRepository
            .getCreatorRankings()
            .filter { !memberService.isBlocked(blockedMemberId = member.id!!, memberId = it.id!!) }
            .map { it.toExplorerSectionCreator(cloudFrontHost) }

        val currentDateTime = LocalDateTime.now()
        val lastMonday = currentDateTime
            .minusWeeks(1)
            .with(TemporalAdjusters.previousOrSame(DayOfWeek.MONDAY))
        val lastSunday = lastMonday
            .plusDays(6)

        val startDateFormatter = DateTimeFormatter.ofPattern("yyyy년 MM월 dd일")
        val endDateFormatter = DateTimeFormatter.ofPattern("MM월 dd일")

        val formattedLastMonday = lastMonday.format(startDateFormatter)
        val formattedLastSunday = lastSunday.format(endDateFormatter)

        val creatorRankingSection = GetExplorerSectionResponse(
            title = "인기 크리에이터",
            coloredTitle = "인기",
            color = "FF5C49",
            desc = "$formattedLastMonday ~ $formattedLastSunday",
            creators = creatorRankings
        )
        sections.add(creatorRankingSection)

        // 새로 시작 (newCreators)
        val newCreators = queryRepository
            .getNewCreators()
            .filter { !memberService.isBlocked(blockedMemberId = member.id!!, memberId = it.id!!) }
            .map { it.toExplorerSectionCreator(cloudFrontHost) }

        val newCreatorsSection = GetExplorerSectionResponse(
            title = "새로 시작",
            coloredTitle = "새로",
            color = "5FD28F",
            creators = newCreators
        )
        sections.add(newCreatorsSection)

        val maleCreatorSection = GetExplorerSectionResponse(
            title = "남자 크리에이터",
            coloredTitle = "남자",
            color = "39abde",
            creators = queryRepository
                .findCreatorByGender(1)
                .filter { !memberService.isBlocked(blockedMemberId = member.id!!, memberId = it.id!!) }
                .map { it.toExplorerSectionCreator(cloudFrontHost) }
        )

        val femaleCreatorSection = GetExplorerSectionResponse(
            title = "여자 크리에이터",
            coloredTitle = "여자",
            color = "ffa517",
            creators = queryRepository
                .findCreatorByGender(0)
                .filter { !memberService.isBlocked(blockedMemberId = member.id!!, memberId = it.id!!) }
                .map { it.toExplorerSectionCreator(cloudFrontHost) }
        )

        val randomInt = Random.nextInt(0, 2)
        if (randomInt == 0) {
            sections.add(femaleCreatorSection)
            sections.add(maleCreatorSection)
        } else {
            sections.add(maleCreatorSection)
            sections.add(femaleCreatorSection)
        }

        return GetExplorerResponse(sections = sections)
    }

    fun getSearchChannel(channel: String, member: Member): List<GetRoomDetailUser> {
        if (channel.length < 2) {
            throw SodaException("두 글자 이상 입력 하셔야 합니다.")
        }

        return queryRepository.getSearchChannel(channel, member.id!!)
            .asSequence()
            .filter { !memberService.isBlocked(blockedMemberId = member.id!!, memberId = it.id!!) }
            .map { GetRoomDetailUser(it, cloudFrontHost) }
            .toList()
    }

    fun getCreatorProfile(creatorId: Long, timezone: String, member: Member): GetCreatorProfileResponse {
        // 크리에이터(유저) 정보
        val creatorAccount = queryRepository.getMember(creatorId) ?: throw SodaException("없는 사용자 입니다.")

        // 차단된 사용자 체크
        val isBlocked = memberService.isBlocked(blockedMemberId = member.id!!, memberId = creatorId)
        if (isBlocked) throw SodaException("${creatorAccount.nickname}님의 요청으로 채널 접근이 제한됩니다.")

        val isCreator = creatorAccount.role == MemberRole.CREATOR

        val notificationUserIds = queryRepository.getNotificationUserIds(creatorId)
        val creatorFollowing = queryRepository.getCreatorFollowing(creatorId = creatorId, memberId = member.id!!)
        val notificationRecipientCount = notificationUserIds.size

        // 후원랭킹
        val memberDonationRanking = if (
            isCreator && (creatorId == member.id!! || creatorAccount.isVisibleDonationRank)
        ) {
            queryRepository.getMemberDonationRanking(
                creatorId,
                10,
                withDonationCan = creatorId == member.id!!
            )
        } else {
            listOf()
        }

        // 추천 크리에이터
        val similarCreatorList = if (isCreator) {
            queryRepository.getSimilarCreatorList(creatorId)
        } else {
            listOf()
        }

        // 라이브
        val liveRoomList = if (isCreator) {
            queryRepository.getLiveRoomList(
                creatorId,
                userMember = member,
                timezone = timezone,
                limit = 3
            )
        } else {
            listOf()
        }

        // 오디오 콘텐츠
        val contentList = if (isCreator) {
            audioContentService.getAudioContentList(
                creatorId = creatorId,
                sortType = SortType.NEWEST,
                member = member,
                offset = 0,
                limit = 3
            ).items
        } else {
            listOf()
        }

        // 공지사항
        val notice = if (isCreator) {
            queryRepository.getNoticeString(creatorId)
        } else {
            ""
        }

        // 커뮤니티
        val communityPostList = if (isCreator) {
            communityService.getCommunityPostList(
                creatorId = creatorId,
                memberId = member.id!!,
                timezone = timezone,
                offset = 0,
                limit = 3,
                isAdult = member.auth != null
            )
        } else {
            listOf()
        }

        // 응원
        val cheers = queryRepository.getCheersList(creatorId, timezone = timezone, offset = 0, limit = 4)

        // 차단한 크리에이터 인지 체크
        val isBlock = memberService.isBlocked(blockedMemberId = creatorId, memberId = member.id!!)

        val activitySummary = if (isCreator) {
            // 활동요약 (라이브 횟수, 라이브 시간, 라이브 참여자, 콘텐츠 수)
            val liveCount = queryRepository.getLiveCount(creatorId) ?: 0
            val liveTime = queryRepository.getLiveTime(creatorId)
            val liveContributorCount = queryRepository.getLiveContributorCount(creatorId) ?: 0
            val contentCount = queryRepository.getContentCount(creatorId) ?: 0
            GetCreatorActivitySummary(
                liveCount = liveCount,
                liveTime = liveTime,
                liveContributorCount = liveContributorCount,
                contentCount = contentCount
            )
        } else {
            GetCreatorActivitySummary(0, 0, 0, 0)
        }

        val seriesList = if (isCreator) {
            seriesService
                .getSeriesList(creatorId = creatorId, member = member)
                .items
        } else {
            listOf()
        }

        return GetCreatorProfileResponse(
            creator = CreatorResponse(
                creatorId = creatorAccount.id!!,
                profileUrl = if (creatorAccount.profileImage != null) {
                    "$cloudFrontHost/${creatorAccount.profileImage}"
                } else {
                    "$cloudFrontHost/profile/default-profile.png"
                },
                nickname = creatorAccount.nickname,
                tags = creatorAccount.tags.asSequence().filter { it.tag.isActive }.map { it.tag.tag }.toList(),
                introduce = creatorAccount.introduce,
                instagramUrl = creatorAccount.instagramUrl,
                youtubeUrl = creatorAccount.youtubeUrl,
                websiteUrl = creatorAccount.websiteUrl,
                blogUrl = creatorAccount.blogUrl,
                isFollow = creatorFollowing?.isFollow ?: false,
                isNotify = creatorFollowing?.isNotify ?: false,
                isNotification = creatorFollowing?.isFollow ?: false,
                notificationRecipientCount = notificationRecipientCount
            ),
            userDonationRanking = memberDonationRanking,
            similarCreatorList = similarCreatorList,
            liveRoomList = liveRoomList,
            contentList = contentList,
            notice = notice,
            communityPostList = communityPostList,
            cheers = cheers,
            activitySummary = activitySummary,
            seriesList = seriesList,
            isBlock = isBlock,
            isCreatorRole = isCreator
        )
    }

    fun getCreatorProfileDonationRanking(
        creatorId: Long,
        pageable: Pageable,
        member: Member
    ): GetDonationAllResponse {
        val currentDate = LocalDate.now().atTime(0, 0, 0)
        val firstDayOfLastWeek = currentDate.with(TemporalAdjusters.previousOrSame(DayOfWeek.MONDAY)).minusDays(7)
        val firstDayOfMonth = currentDate.with(TemporalAdjusters.firstDayOfMonth())

        return GetDonationAllResponse(
            accumulatedCansToday = if (creatorId == member.id!!) {
                queryRepository.getDonationCoinsDateRange(
                    creatorId,
                    currentDate,
                    currentDate.plusDays(1)
                ) ?: 0
            } else {
                0
            },
            accumulatedCansLastWeek = if (creatorId == member.id!!) {
                queryRepository.getDonationCoinsDateRange(
                    creatorId,
                    firstDayOfLastWeek,
                    firstDayOfLastWeek.plusDays(7)
                ) ?: 0
            } else {
                0
            },
            accumulatedCansThisMonth = if (creatorId == member.id!!) {
                queryRepository.getDonationCoinsDateRange(
                    creatorId,
                    firstDayOfMonth,
                    currentDate
                ) ?: 0
            } else {
                0
            },
            isVisibleDonationRank = if (creatorId == member.id!!) {
                queryRepository.getVisibleDonationRank(creatorId)
            } else {
                false
            },
            totalCount = queryRepository.getMemberDonationRankingTotal(creatorId),
            userDonationRanking = queryRepository.getMemberDonationRanking(
                creatorId,
                offset = pageable.offset,
                limit = pageable.pageSize.toLong(),
                withDonationCan = creatorId == member.id!!
            )
        )
    }

    fun getFollowerList(
        creatorId: Long,
        member: Member,
        pageable: Pageable
    ): GetFollowerListResponse {
        val totalCount = queryRepository.getFollowerListTotalCount(creatorId)

        val followerList = queryRepository.getFollowerList(creatorId, pageable.offset, pageable.pageSize.toLong())
            .map {
                val creatorFollowing = if (it.role == MemberRole.CREATOR) {
                    queryRepository.getCreatorFollowing(
                        creatorId = it.userId,
                        memberId = member.id!!
                    )
                } else {
                    null
                }

                GetFollowerListResponseItem(
                    userId = it.userId,
                    profileImage = it.profileImage,
                    nickname = it.nickname,
                    isFollow = creatorFollowing?.isFollow ?: if (it.role == MemberRole.CREATOR) {
                        false
                    } else {
                        null
                    },
                    isNotify = creatorFollowing?.isNotify ?: if (it.role == MemberRole.CREATOR) {
                        false
                    } else {
                        null
                    }
                )
            }

        return GetFollowerListResponse(totalCount = totalCount, items = followerList)
    }

    @Transactional
    fun writeCheers(request: PostWriteCheersRequest, member: Member) {
        val creator = queryRepository.getMember(request.creatorId) ?: throw SodaException("없는 사용자 입니다.")

        val isBlocked = memberService.isBlocked(blockedMemberId = member.id!!, memberId = request.creatorId)
        if (isBlocked) throw SodaException("${creator.nickname}님의 요청으로 팬토크 작성이 제한됩니다.")

        val cheers = CreatorCheers(cheers = request.content)
        cheers.member = member
        cheers.creator = creator

        val parent = if (request.parentId != null) {
            queryRepository.getCreatorCheers(request.parentId)
        } else {
            null
        }

        if (parent != null) {
            cheers.parent = parent
        }

        cheersRepository.save(cheers)
    }

    fun getCreatorProfileCheers(
        creatorId: Long,
        timezone: String,
        pageable: Pageable
    ): GetCheersResponse {
        return queryRepository.getCheersList(
            creatorId = creatorId,
            timezone = timezone,
            offset = pageable.offset,
            limit = pageable.pageSize.toLong()
        )
    }

    @Transactional
    fun modifyCheers(request: PutWriteCheersRequest, member: Member) {
        val cheers = queryRepository.getCheers(request.cheersId)
            ?: throw SodaException("잘못된 요청입니다.")

        if (cheers.member!!.id!! == member.id!!) {
            if (request.content != null) {
                cheers.cheers = request.content
            }
        }

        if (
            cheers.creator!!.id!! == member.id!! ||
            cheers.member!!.id!! == member.id!!
        ) {
            if (request.isActive != null) {
                cheers.isActive = request.isActive
            }
        }
    }

    @Transactional
    fun saveNotice(member: Member, notice: String) {
        var channelNotice = queryRepository.getNotice(creatorId = member.id!!)

        if (channelNotice == null) {
            channelNotice = ChannelNotice(notice)
            channelNotice.member = member
            noticeRepository.save(channelNotice)
        } else {
            channelNotice.notice = notice
        }

        applicationEventPublisher.publishEvent(
            FcmEvent(
                type = FcmEventType.CHANGE_NOTICE,
                title = member.nickname,
                message = "새 글이 등록되었습니다.",
                creatorId = member.id!!
            )
        )
    }
}
