package kr.co.vividnext.sodalive.live.room

import com.querydsl.core.types.Projections
import com.querydsl.core.types.dsl.CaseBuilder
import com.querydsl.core.types.dsl.Expressions
import com.querydsl.core.types.dsl.NumberExpression
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.can.use.CanUsage
import kr.co.vividnext.sodalive.can.use.QUseCan.useCan
import kr.co.vividnext.sodalive.can.use.QUseCanCalculate.useCanCalculate
import kr.co.vividnext.sodalive.live.room.QLiveRoom.liveRoom
import kr.co.vividnext.sodalive.live.room.QQuarterLiveRankings.quarterLiveRankings
import kr.co.vividnext.sodalive.live.room.donation.GetLiveRoomDonationItem
import kr.co.vividnext.sodalive.live.room.donation.QGetLiveRoomDonationItem
import kr.co.vividnext.sodalive.live.room.like.GetLiveRoomHeartListItem
import kr.co.vividnext.sodalive.live.room.like.QGetLiveRoomHeartListItem
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository
import java.time.LocalDateTime
import java.time.ZoneId

@Repository
interface LiveRoomRepository : JpaRepository<LiveRoom, Long>, LiveRoomQueryRepository

interface LiveRoomQueryRepository {
    fun getLiveRoomListNow(
        offset: Long,
        limit: Long,
        timezone: String,
        memberId: Long,
        isCreator: Boolean,
        isAdult: Boolean
    ): List<LiveRoom>

    fun getLiveRoomListReservationWithDate(
        date: LocalDateTime,
        offset: Long,
        limit: Long,
        memberId: Long,
        isCreator: Boolean,
        isAdult: Boolean
    ): List<LiveRoom>

    fun getLiveRoomListReservationWithoutDate(
        timezone: String,
        memberId: Long,
        isCreator: Boolean,
        isAdult: Boolean
    ): List<LiveRoom>

    fun getLiveRoom(id: Long): LiveRoom?
    fun getLiveRoomAndAccountId(roomId: Long, memberId: Long): LiveRoom?
    fun getRecentRoomInfo(memberId: Long): GetRecentRoomInfoResponse?
    fun getDonationTotal(roomId: Long): Int?
    fun getDonationList(roomId: Long, isLiveCreator: Boolean, memberId: Long): List<GetLiveRoomDonationItem>
    fun getRoomActiveAndChannelNameIsNotNull(memberId: Long): List<LiveRoom>
    fun getActiveRoomIdList(memberId: Long): Int
    fun getTotalHeartCount(roomId: Long): Int?
    fun getLiveRoomCreatorId(roomId: Long): Long?
    fun getHeartList(roomId: Long): List<GetLiveRoomHeartListItem>
}

class LiveRoomQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory,

    @Value("\${cloud.aws.cloud-front.host}")
    private val cloudFrontHost: String
) : LiveRoomQueryRepository {
    override fun getLiveRoomListNow(
        offset: Long,
        limit: Long,
        timezone: String,
        memberId: Long,
        isCreator: Boolean,
        isAdult: Boolean
    ): List<LiveRoom> {
        var where = liveRoom.channelName.isNotNull
            .and(liveRoom.channelName.isNotEmpty)
            .and(liveRoom.isActive.isTrue)
            .and(liveRoom.member.isNotNull)

        if (!isAdult) {
            where = where.and(liveRoom.isAdult.isFalse)
        }

        if (isCreator) {
            where = where.and(
                liveRoom.isAvailableJoinCreator.isTrue
                    .or(liveRoom.member.id.eq(memberId))
            )
        }

        return queryFactory
            .selectFrom(liveRoom)
            .innerJoin(liveRoom.member, member)
            .leftJoin(quarterLiveRankings).on(liveRoom.id.eq(quarterLiveRankings.roomId))
            .where(where)
            .offset(offset)
            .limit(limit)
            .orderBy(
                quarterLiveRankings.totalCan.coalesce(0).desc(),
                quarterLiveRankings.totalParticipants.coalesce(0).desc(),
                liveRoom.beginDateTime.desc(),
                liveRoom.member.createdAt.desc()
            )
            .fetch()
    }

    override fun getLiveRoomListReservationWithDate(
        date: LocalDateTime,
        offset: Long,
        limit: Long,
        memberId: Long,
        isCreator: Boolean,
        isAdult: Boolean
    ): List<LiveRoom> {
        var where = liveRoom.beginDateTime.goe(date)
            .and(liveRoom.beginDateTime.lt(date.plusDays(1)))
            .and(
                liveRoom.channelName.isNull
                    .or(liveRoom.channelName.isEmpty)
            )
            .and(liveRoom.isActive.isTrue)
            .and(liveRoom.member.isNotNull)

        if (!isAdult) {
            where = where.and(liveRoom.isAdult.isFalse)
        }

        if (isCreator) {
            where = where.and(
                liveRoom.isAvailableJoinCreator.isTrue
                    .or(liveRoom.member.id.eq(memberId))
            )
        }

        return queryFactory
            .selectFrom(liveRoom)
            .innerJoin(liveRoom.member, member)
            .where(where)
            .offset(offset)
            .limit(limit)
            .orderBy(liveRoom.beginDateTime.asc())
            .fetch()
    }

    override fun getLiveRoomListReservationWithoutDate(
        timezone: String,
        memberId: Long,
        isCreator: Boolean,
        isAdult: Boolean
    ): List<LiveRoom> {
        var where = liveRoom.beginDateTime.gt(
            LocalDateTime.now()
                .atZone(ZoneId.of(timezone))
                .withZoneSameInstant(ZoneId.of("UTC"))
                .toLocalDateTime()
        )
            .and(
                liveRoom.channelName.isNull
                    .or(liveRoom.channelName.isEmpty)
            )
            .and(liveRoom.isActive.isTrue)
            .and(liveRoom.member.isNotNull)

        if (!isAdult) {
            where = where.and(liveRoom.isAdult.isFalse)
        }

        if (isCreator) {
            where = where.and(
                liveRoom.isAvailableJoinCreator.isTrue
                    .or(liveRoom.member.id.eq(memberId))
            )
        }

        return queryFactory
            .selectFrom(liveRoom)
            .innerJoin(liveRoom.member, member)
            .limit(10)
            .where(where)
            .orderBy(
                CaseBuilder()
                    .`when`(member.id.eq(memberId)).then(1)
                    .otherwise(2)
                    .asc(),
                liveRoom.beginDateTime.asc()
            )
            .fetch()
    }

    override fun getLiveRoom(id: Long): LiveRoom? {
        return queryFactory
            .selectFrom(liveRoom)
            .innerJoin(liveRoom.member, member)
            .where(
                liveRoom.id.eq(id)
                    .and(liveRoom.isActive.isTrue)
            )
            .fetchFirst()
    }

    override fun getLiveRoomAndAccountId(roomId: Long, memberId: Long): LiveRoom? {
        return queryFactory
            .selectFrom(liveRoom)
            .innerJoin(liveRoom.member, member)
            .where(
                liveRoom.id.eq(roomId)
                    .and(liveRoom.isActive.isTrue)
                    .and(member.id.eq(memberId))
            )
            .fetchFirst()
    }

    override fun getRecentRoomInfo(memberId: Long): GetRecentRoomInfoResponse? {
        return queryFactory
            .select(
                Projections.constructor(
                    GetRecentRoomInfoResponse::class.java,
                    liveRoom.title,
                    liveRoom.notice,
                    liveRoom.coverImage.prepend("/").prepend(cloudFrontHost),
                    liveRoom.coverImage,
                    liveRoom.numberOfPeople
                )
            )
            .from(liveRoom)
            .where(liveRoom.member.id.eq(memberId))
            .orderBy(liveRoom.id.desc())
            .limit(1)
            .fetchFirst()
    }

    override fun getDonationTotal(roomId: Long): Int? {
        val where = liveRoom.id.eq(roomId)
            .and(useCan.canUsage.eq(CanUsage.DONATION).or(useCan.canUsage.eq(CanUsage.SPIN_ROULETTE)))
            .and(useCan.isRefund.isFalse)
            .and(useCan.isSecret.isFalse)

        return queryFactory
            .select(useCanCalculate.can.sum())
            .from(useCanCalculate)
            .innerJoin(useCanCalculate.useCan, useCan)
            .innerJoin(useCan.room, liveRoom)
            .where(where)
            .fetchOne()
    }

    override fun getDonationList(roomId: Long, isLiveCreator: Boolean, memberId: Long): List<GetLiveRoomDonationItem> {
        val where = liveRoom.id.eq(roomId)
            .and(useCan.canUsage.eq(CanUsage.DONATION).or(useCan.canUsage.eq(CanUsage.SPIN_ROULETTE)))
            .and(useCan.isRefund.isFalse)

        val sortExpression: NumberExpression<Int> = if (isLiveCreator) {
            useCan.can.add(useCan.rewardCan)
        } else {
            CaseBuilder()
                .`when`(useCan.isSecret.isFalse)
                .then(useCan.can.add(useCan.rewardCan))
                .otherwise(0)
        }

        return queryFactory
            .select(
                QGetLiveRoomDonationItem(
                    member.profileImage
                        .coalesce("profile/default-profile.png")
                        .prepend("/")
                        .prepend(cloudFrontHost),
                    member.nickname,
                    member.id.coalesce(0),
                    Expressions.asNumber(
                        CaseBuilder()
                            .`when`(useCan.isSecret.isFalse)
                            .then(useCan.can.add(useCan.rewardCan))
                            .otherwise(0)
                    ).sum(),
                    Expressions.asNumber(
                        CaseBuilder()
                            .`when`(Expressions.asBoolean(isLiveCreator).isTrue.or(useCan.member.id.eq(memberId)))
                            .then(
                                Expressions.asNumber(
                                    CaseBuilder()
                                        .`when`(useCan.isSecret.isTrue)
                                        .then(useCan.can.add(useCan.rewardCan))
                                        .otherwise(0)
                                ).sum()
                            )
                            .otherwise(0)
                    )
                )
            )
            .from(useCan)
            .innerJoin(useCan.room, liveRoom)
            .innerJoin(useCan.member, member)
            .groupBy(useCan.member.id)
            .where(where)
            .orderBy(sortExpression.sum().desc())
            .fetch()
    }

    override fun getRoomActiveAndChannelNameIsNotNull(memberId: Long): List<LiveRoom> {
        return queryFactory
            .selectFrom(liveRoom)
            .where(
                liveRoom.isActive.isTrue
                    .and(liveRoom.channelName.isNotNull)
                    .and(liveRoom.member.id.eq(memberId))
            )
            .fetch()
    }

    override fun getActiveRoomIdList(memberId: Long): Int {
        return queryFactory
            .select(liveRoom.id)
            .from(liveRoom)
            .where(
                liveRoom.isActive.isTrue
                    .and(liveRoom.channelName.isNull)
                    .and(liveRoom.member.id.eq(memberId))
            )
            .fetch()
            .size
    }

    override fun getTotalHeartCount(roomId: Long): Int? {
        val where = liveRoom.id.eq(roomId)
            .and(useCan.canUsage.eq(CanUsage.HEART))
            .and(useCan.isRefund.isFalse)

        return queryFactory
            .select(useCanCalculate.can.sum())
            .from(useCanCalculate)
            .innerJoin(useCanCalculate.useCan, useCan)
            .innerJoin(useCan.room, liveRoom)
            .where(where)
            .fetchOne()
    }

    override fun getLiveRoomCreatorId(roomId: Long): Long? {
        return queryFactory
            .select(liveRoom.member.id)
            .from(liveRoom)
            .where(
                liveRoom.isActive.isTrue
                    .and(liveRoom.id.eq(roomId))
            )
            .fetchFirst()
    }

    override fun getHeartList(roomId: Long): List<GetLiveRoomHeartListItem> {
        val where = liveRoom.id.eq(roomId)
            .and(useCan.canUsage.eq(CanUsage.HEART))
            .and(useCan.isRefund.isFalse)

        return queryFactory
            .select(
                QGetLiveRoomHeartListItem(
                    member.profileImage
                        .coalesce("profile/default-profile.png")
                        .prepend("/")
                        .prepend(cloudFrontHost),
                    member.nickname,
                    useCan.can.add(useCan.rewardCan).sum()
                )
            )
            .from(useCan)
            .innerJoin(useCan.room, liveRoom)
            .innerJoin(useCan.member, member)
            .groupBy(useCan.member.id)
            .where(where)
            .orderBy(useCan.can.add(useCan.rewardCan).sum().desc())
            .fetch()
    }
}
