package kr.co.vividnext.sodalive.rank

import com.querydsl.core.types.dsl.Expressions
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.admin.content.series.genre.QSeriesGenre.seriesGenre
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.comment.QAudioContentComment.audioContentComment
import kr.co.vividnext.sodalive.content.like.QAudioContentLike.audioContentLike
import kr.co.vividnext.sodalive.content.main.ContentCreatorResponse
import kr.co.vividnext.sodalive.content.main.GetAudioContentRankingItem
import kr.co.vividnext.sodalive.content.main.QContentCreatorResponse
import kr.co.vividnext.sodalive.content.main.QGetAudioContentRankingItem
import kr.co.vividnext.sodalive.content.order.QOrder.order
import kr.co.vividnext.sodalive.content.theme.QAudioContentTheme.audioContentTheme
import kr.co.vividnext.sodalive.creator.admin.content.series.QSeries.series
import kr.co.vividnext.sodalive.creator.admin.content.series.QSeriesContent.seriesContent
import kr.co.vividnext.sodalive.creator.admin.content.series.Series
import kr.co.vividnext.sodalive.creator.admin.content.series.SeriesState
import kr.co.vividnext.sodalive.explorer.QCreatorRanking.creatorRanking
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.MemberRole
import kr.co.vividnext.sodalive.member.QMember.member
import kr.co.vividnext.sodalive.member.block.QBlockMember.blockMember
import org.springframework.beans.factory.annotation.Value
import org.springframework.stereotype.Repository
import java.time.LocalDateTime

@Repository
class RankingRepository(
    private val queryFactory: JPAQueryFactory,

    @Value("\${cloud.aws.cloud-front.host}")
    private val imageHost: String
) {
    fun getCreatorRankings(): List<Member> {
        return queryFactory
            .select(member)
            .from(creatorRanking)
            .innerJoin(creatorRanking.member, member)
            .orderBy(creatorRanking.ranking.asc())
            .fetch()
    }

    fun getAudioContentRanking(
        memberId: Long,
        isAdult: Boolean,
        startDate: LocalDateTime,
        endDate: LocalDateTime,
        offset: Long,
        limit: Long,
        sortType: String,
        theme: String = ""
    ): List<GetAudioContentRankingItem> {
        val blockMemberCondition = blockMember.member.id.eq(member.id)
            .and(blockMember.isActive.isTrue)
            .and(blockMember.blockedMember.id.eq(memberId))

        var where = audioContent.isActive.isTrue
            .and(audioContent.member.isActive.isTrue)
            .and(audioContent.member.isNotNull)
            .and(audioContent.member.role.eq(MemberRole.CREATOR))
            .and(audioContent.duration.isNotNull)
            .and(audioContentTheme.isActive.isTrue)
            .and(audioContent.limited.isNull)
            .and(blockMember.id.isNull)

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        }

        if (theme.isNotBlank()) {
            where = where.and(audioContentTheme.theme.eq(theme))
        }

        var select = queryFactory
            .select(
                QGetAudioContentRankingItem(
                    audioContent.id,
                    audioContent.title,
                    audioContent.coverImage.prepend("/").prepend(imageHost),
                    audioContentTheme.theme,
                    audioContent.price,
                    audioContent.duration,
                    member.id,
                    member.nickname,
                    member.profileImage.prepend("/").prepend(imageHost)
                )
            )

        select = when (sortType) {
            "후원" -> {
                select
                    .from(audioContentComment)
                    .innerJoin(audioContentComment.audioContent, audioContent)
                    .innerJoin(audioContent.member, member)
                    .innerJoin(audioContent.theme, audioContentTheme)
                    .leftJoin(blockMember).on(blockMemberCondition)
                    .where(
                        where
                            .and(audioContentComment.isActive.isTrue)
                            .and(audioContentComment.donationCan.gt(0))
                            .and(audioContentComment.createdAt.goe(startDate))
                            .and(audioContentComment.createdAt.lt(endDate))
                    )
                    .groupBy(audioContent.id)
                    .orderBy(audioContentComment.donationCan.sum().desc(), audioContent.createdAt.asc())
            }

            "댓글" -> {
                select
                    .from(audioContentComment)
                    .innerJoin(audioContentComment.audioContent, audioContent)
                    .innerJoin(audioContent.member, member)
                    .innerJoin(audioContent.theme, audioContentTheme)
                    .leftJoin(blockMember).on(blockMemberCondition)
                    .where(
                        where
                            .and(audioContentComment.isActive.isTrue)
                            .and(audioContentComment.createdAt.goe(startDate))
                            .and(audioContentComment.createdAt.lt(endDate))
                    )
                    .groupBy(audioContentComment.audioContent.id)
                    .orderBy(audioContentComment.id.count().desc(), audioContent.createdAt.asc())
            }

            "좋아요" -> {
                select
                    .from(audioContentLike)
                    .innerJoin(audioContentLike.audioContent, audioContent)
                    .innerJoin(audioContent.member, member)
                    .innerJoin(audioContent.theme, audioContentTheme)
                    .leftJoin(blockMember).on(blockMemberCondition)
                    .where(
                        where
                            .and(audioContentLike.isActive.isTrue)
                            .and(audioContentLike.createdAt.goe(startDate))
                            .and(audioContentLike.createdAt.lt(endDate))
                    )
                    .groupBy(audioContentLike.audioContent.id)
                    .orderBy(audioContentLike.id.count().desc(), audioContent.createdAt.asc())
            }

            else -> {
                select
                    .from(order)
                    .innerJoin(order.audioContent, audioContent)
                    .innerJoin(audioContent.member, member)
                    .innerJoin(audioContent.theme, audioContentTheme)
                    .leftJoin(blockMember).on(blockMemberCondition)
                    .where(
                        where
                            .and(order.isActive.isTrue)
                            .and(order.createdAt.goe(startDate))
                            .and(order.createdAt.lt(endDate))
                    )
                    .groupBy(audioContent.id)
                    .orderBy(order.can.sum().desc(), audioContent.createdAt.asc())
            }
        }

        return select
            .offset(offset)
            .limit(limit)
            .fetch()
    }

    fun getSeriesRanking(
        memberId: Long,
        isAdult: Boolean,
        startDate: LocalDateTime,
        endDate: LocalDateTime
    ): List<Series> {
        val blockMemberCondition = blockMember.member.id.eq(member.id)
            .and(blockMember.isActive.isTrue)
            .and(blockMember.blockedMember.id.eq(memberId))

        var where = series.isActive.isTrue
            .and(audioContent.isActive.isTrue)
            .and(member.isActive.isTrue)
            .and(member.isNotNull)
            .and(member.role.eq(MemberRole.CREATOR))
            .and(audioContent.duration.isNotNull)
            .and(audioContent.limited.isNull)
            .and(blockMember.id.isNull)
            .and(order.isActive.isTrue)
            .and(order.createdAt.goe(startDate))
            .and(order.createdAt.lt(endDate))

        if (!isAdult) {
            where = where.and(series.isAdult.isFalse)
        }

        return queryFactory
            .select(series)
            .from(seriesContent)
            .innerJoin(seriesContent.series, series)
            .innerJoin(seriesContent.content, audioContent)
            .innerJoin(series.member, member)
            .leftJoin(order).on(audioContent.id.eq(order.audioContent.id))
            .leftJoin(blockMember).on(blockMemberCondition)
            .where(where)
            .groupBy(series.id)
            .orderBy(
                order.can.sum().desc(),
                Expressions.numberTemplate(Double::class.java, "function('rand')").asc()
            )
            .offset(0)
            .limit(10)
            .fetch()
    }

    fun getCompleteSeriesRankingTotalCount(memberId: Long, isAdult: Boolean): Int {
        val blockMemberCondition = blockMember.member.id.eq(member.id)
            .and(blockMember.isActive.isTrue)
            .and(blockMember.blockedMember.id.eq(memberId))

        var where = series.isActive.isTrue
            .and(series.state.eq(SeriesState.COMPLETE))
            .and(audioContent.isActive.isTrue)
            .and(member.isActive.isTrue)
            .and(member.isNotNull)
            .and(member.role.eq(MemberRole.CREATOR))
            .and(audioContent.duration.isNotNull)
            .and(audioContent.limited.isNull)
            .and(blockMember.id.isNull)

        if (!isAdult) {
            where = where.and(series.isAdult.isFalse)
        }

        return queryFactory
            .select(series.id)
            .from(seriesContent)
            .innerJoin(seriesContent.series, series)
            .innerJoin(seriesContent.content, audioContent)
            .innerJoin(series.member, member)
            .leftJoin(blockMember).on(blockMemberCondition)
            .where(where)
            .groupBy(series.id)
            .fetch()
            .size
    }

    fun getCompleteSeriesRanking(
        memberId: Long,
        isAdult: Boolean,
        startDate: LocalDateTime,
        endDate: LocalDateTime,
        offset: Long,
        limit: Long
    ): List<Series> {
        val blockMemberCondition = blockMember.member.id.eq(member.id)
            .and(blockMember.isActive.isTrue)
            .and(blockMember.blockedMember.id.eq(memberId))

        val orderCondition = order.isActive.isTrue
            .and(audioContent.id.eq(order.audioContent.id))
            .and(order.createdAt.goe(startDate))
            .and(order.createdAt.lt(endDate))

        var where = series.isActive.isTrue
            .and(series.state.eq(SeriesState.COMPLETE))
            .and(audioContent.isActive.isTrue)
            .and(member.isActive.isTrue)
            .and(member.isNotNull)
            .and(member.role.eq(MemberRole.CREATOR))
            .and(audioContent.duration.isNotNull)
            .and(audioContent.limited.isNull)
            .and(blockMember.id.isNull)

        if (!isAdult) {
            where = where.and(series.isAdult.isFalse)
        }

        return queryFactory
            .select(series)
            .from(seriesContent)
            .innerJoin(seriesContent.series, series)
            .innerJoin(seriesContent.content, audioContent)
            .innerJoin(series.member, member)
            .leftJoin(order).on(orderCondition)
            .leftJoin(blockMember).on(blockMemberCondition)
            .where(where)
            .groupBy(series.id)
            .orderBy(order.id.count().desc(), series.id.desc())
            .offset(offset)
            .limit(limit)
            .fetch()
    }

    fun getSeriesAllRankingByGenre(memberId: Long, isAdult: Boolean, genreId: Long): List<Series> {
        val blockMemberCondition = blockMember.member.id.eq(member.id)
            .and(blockMember.isActive.isTrue)
            .and(blockMember.blockedMember.id.eq(memberId))

        var where = series.isActive.isTrue
            .and(seriesGenre.id.eq(genreId))
            .and(audioContent.isActive.isTrue)
            .and(member.isActive.isTrue)
            .and(member.isNotNull)
            .and(member.role.eq(MemberRole.CREATOR))
            .and(audioContent.duration.isNotNull)
            .and(audioContent.limited.isNull)
            .and(blockMember.id.isNull)
            .and(order.isActive.isTrue)

        if (!isAdult) {
            where = where.and(series.isAdult.isFalse)
        }

        return queryFactory
            .select(series)
            .from(seriesContent)
            .innerJoin(seriesContent.series, series)
            .innerJoin(series.genre, seriesGenre)
            .innerJoin(seriesContent.content, audioContent)
            .innerJoin(series.member, member)
            .leftJoin(order).on(audioContent.id.eq(order.audioContent.id))
            .leftJoin(blockMember).on(blockMemberCondition)
            .where(where)
            .groupBy(series.id)
            .orderBy(
                order.id.countDistinct().desc(),
                Expressions.numberTemplate(Double::class.java, "function('rand')").asc()
            )
            .offset(0)
            .limit(20)
            .fetch()
    }

    fun fetchCreatorBySellContentCountRankTop20(
        memberId: Long,
        startDate: LocalDateTime,
        endDate: LocalDateTime
    ): List<ContentCreatorResponse> {
        val blockMemberCondition = blockMember.member.id.eq(member.id)
            .and(blockMember.isActive.isTrue)
            .and(blockMember.blockedMember.id.eq(memberId))

        val ordersCondition = order.audioContent.id.eq(audioContent.id)
            .and(order.isActive.isTrue)
            .and(order.createdAt.goe(startDate))
            .and(order.createdAt.lt(endDate))

        val memberCondition = member.isActive.isTrue
            .and(member.role.eq(MemberRole.CREATOR))
            .and(member.id.eq(audioContent.member.id))

        val where = audioContent.isActive.isTrue
            .and(audioContent.price.gt(0))
            .and(audioContent.duration.isNotNull)
            .and(audioContent.limited.isNull)
            .and(blockMember.id.isNull)

        return queryFactory
            .select(
                QContentCreatorResponse(
                    member.id,
                    member.nickname,
                    member.profileImage.prepend("/").prepend(imageHost)
                )
            )
            .from(audioContent)
            .innerJoin(member).on(memberCondition)
            .leftJoin(order).on(ordersCondition)
            .leftJoin(blockMember).on(blockMemberCondition)
            .where(where)
            .groupBy(member.id)
            .having(audioContent.id.count().goe(4))
            .orderBy(order.id.count().desc(), member.id.desc())
            .offset(0)
            .limit(20)
            .fetch()
    }

    fun fetchCreatorContentBySalesCountTop4(
        creatorId: Long,
        isAdult: Boolean,
        theme: String
    ): List<GetAudioContentRankingItem> {
        var where = member.isActive.isTrue
            .and(member.role.eq(MemberRole.CREATOR))
            .and(audioContent.isActive.isTrue)
            .and(audioContent.duration.isNotNull)
            .and(audioContent.limited.isNull)
            .and(audioContentTheme.isActive.isTrue)
            .and(order.isActive.isTrue)
            .and(member.id.eq(creatorId))

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        }

        if (theme.isNotBlank()) {
            where = where.and(audioContentTheme.theme.eq(theme))
        }

        return queryFactory
            .select(
                QGetAudioContentRankingItem(
                    audioContent.id,
                    audioContent.title,
                    audioContent.coverImage.prepend("/").prepend(imageHost),
                    audioContentTheme.theme,
                    audioContent.price,
                    audioContent.duration,
                    member.id,
                    member.nickname,
                    member.profileImage.prepend("/").prepend(imageHost)
                )
            )
            .from(order)
            .innerJoin(order.audioContent, audioContent)
            .innerJoin(audioContent.theme, audioContentTheme)
            .innerJoin(audioContent.member, member)
            .where(where)
            .groupBy(audioContent.id)
            .orderBy(order.id.count().desc())
            .offset(0)
            .limit(4)
            .fetch()
    }

    fun fetchCreatorBySeriesRevenueRankTop20(
        memberId: Long,
        startDate: LocalDateTime,
        endDate: LocalDateTime
    ): List<ContentCreatorResponse> {
        val blockMemberCondition = blockMember.member.id.eq(member.id)
            .and(blockMember.isActive.isTrue)
            .and(blockMember.blockedMember.id.eq(memberId))

        val ordersCondition = order.audioContent.id.eq(audioContent.id)
            .and(order.isActive.isTrue)
            .and(order.createdAt.goe(startDate))
            .and(order.createdAt.lt(startDate))

        val where = member.isActive.isTrue
            .and(member.role.eq(MemberRole.CREATOR))
            .and(series.isActive.isTrue)
            .and(audioContent.isActive.isTrue)
            .and(audioContent.duration.isNotNull)
            .and(audioContent.limited.isNull)
            .and(blockMember.id.isNull)

        return queryFactory
            .select(
                QContentCreatorResponse(
                    member.id,
                    member.nickname,
                    member.profileImage.prepend("/").prepend(imageHost)
                )
            )
            .from(seriesContent)
            .innerJoin(seriesContent.series, series)
            .innerJoin(seriesContent.content, audioContent)
            .innerJoin(series.member, member)
            .leftJoin(order).on(ordersCondition)
            .leftJoin(blockMember).on(blockMemberCondition)
            .where(where)
            .groupBy(member.id)
            .having(series.id.countDistinct().goe(3))
            .orderBy(
                order.can.sum().desc(),
                Expressions.numberTemplate(Double::class.java, "function('rand')").asc()
            )
            .offset(0)
            .limit(20)
            .fetch()
    }

    fun fetchCreatorSeriesBySales(creatorId: Long, isAdult: Boolean): List<Series> {
        var where = member.isActive.isTrue
            .and(member.role.eq(MemberRole.CREATOR))
            .and(series.isActive.isTrue)
            .and(audioContent.isActive.isTrue)
            .and(audioContent.duration.isNotNull)
            .and(audioContent.limited.isNull)
            .and(order.isActive.isTrue)
            .and(member.id.eq(creatorId))

        if (!isAdult) {
            where = where.and(series.isAdult.isFalse)
        }

        return queryFactory
            .select(series)
            .from(seriesContent)
            .innerJoin(seriesContent.series, series)
            .innerJoin(seriesContent.content, audioContent)
            .innerJoin(series.member, member)
            .leftJoin(order).on(audioContent.id.eq(order.audioContent.id))
            .where(where)
            .groupBy(series.id)
            .orderBy(
                order.can.sum().desc(),
                Expressions.numberTemplate(Double::class.java, "function('rand')").asc()
            )
            .offset(0)
            .limit(10)
            .fetch()
    }

    fun fetchFreeContentByCreatorIdTop4(creatorId: Long, isAdult: Boolean): List<GetAudioContentRankingItem> {
        var where = member.isActive.isTrue
            .and(member.id.eq(creatorId))
            .and(member.role.eq(MemberRole.CREATOR))
            .and(audioContent.isActive.isTrue)
            .and(audioContent.duration.isNotNull)
            .and(audioContent.limited.isNull)
            .and(audioContent.price.loe(0))

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        }

        return queryFactory
            .select(
                QGetAudioContentRankingItem(
                    audioContent.id,
                    audioContent.title,
                    audioContent.coverImage.prepend("/").prepend(imageHost),
                    audioContentTheme.theme,
                    audioContent.price,
                    audioContent.duration,
                    member.id,
                    member.nickname,
                    member.profileImage.prepend("/").prepend(imageHost)
                )
            )
            .from(audioContent)
            .innerJoin(audioContent.theme, audioContentTheme)
            .innerJoin(audioContent.member, member)
            .where(where)
            .orderBy(audioContent.playCount.desc())
            .offset(0)
            .limit(4)
            .fetch()
    }
}
